"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHttpsAgent = exports.createHttpAgent = exports.createAgent = void 0;
/*
 * Copyright (c) AXA Group Operations Spain S.A.
 *
 * Licensed under the AXA Group Operations Spain S.A. License (the "License");
 * you may not use this file except in compliance with the License.
 * A copy of the License can be found in the LICENSE.TXT file distributed
 * together with this file.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const http_agent_1 = require("./http-agent");
const https_agent_1 = require("./https-agent");
const utils_1 = require("./utils");
/**
 * Get the needed proxy agent depending of the given proxy options and target.
 * Also try to gets the proxy from the http/s_proxy env variables
 * @param {string} target - The target to proxy
 * @param {NativeAgentOptions} [options]
 * @returns {HttpsAgent|HttpAgent} The agent depending on the target.
 * @example
 * const { createAgent } = require('@axa/native-proxy-agent');
 * const got = require('got');
 *
 * got('http://myhost.com', { agent : createAgent('http://myhost.com') });
 *
 */
function createAgent(target, options = {}) {
    if (typeof target !== 'string') {
        throw new Error('Target must be defined.');
    }
    if (options.proxy === undefined) {
        const proxy = (0, utils_1.getProxy)();
        if (proxy) {
            Object.assign(options, {
                proxy
            });
        }
    }
    Object.assign(options, { target });
    const url = new URL(target);
    if (url.protocol === 'https:') {
        return new https_agent_1.HttpsAgent(options);
    }
    if (url.protocol === 'http:') {
        return new http_agent_1.HttpAgent(options);
    }
    throw new Error('Target protocol not supported, should be https or http.');
}
exports.createAgent = createAgent;
/**
 * Get the needed proxy agent depending of the given proxy options for an http request
 * Also try to gets the proxy from the http/s_proxy env variables
 * @param {NativeAgentOptions} [options]
 * @returns {HttpsAgent|HttpAgent} The agent depending on the target.
 * @example
 * const { createHttpAgent } = require('@axa/native-proxy-agent');
 * const got = require('got');
 *
 * got('http://myhost.com', { agent : createHttpAgent() });
 *
 */
function createHttpAgent(options = {}) {
    if (options.proxy === undefined) {
        const proxy = (0, utils_1.getProxy)();
        if (proxy) {
            Object.assign(options, {
                proxy
            });
        }
    }
    return new http_agent_1.HttpAgent(options);
}
exports.createHttpAgent = createHttpAgent;
/**
 * Get the needed proxy agent depending of the given proxy options for an https request
 * Also try to gets the proxy from the http/s_proxy env variables
 * @param {NativeAgentOptions} [options]
 * @returns {HttpsAgent|HttpAgent} The agent depending on the target.
 * @example
 * const { createHttpAgent } = require('@axa/native-proxy-agent');
 * const got = require('got');
 *
 * got('http://myhost.com', { agent : createHttpAgent() });
 *
 */
function createHttpsAgent(options = {}) {
    if (options.proxy === undefined) {
        const proxy = (0, utils_1.getProxy)();
        if (proxy) {
            Object.assign(options, {
                proxy
            });
        }
    }
    return new https_agent_1.HttpsAgent(options);
}
exports.createHttpsAgent = createHttpsAgent;
//# sourceMappingURL=agent.js.map