"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHttpsForeverAgent = exports.createHttpForeverAgent = exports.createForeverAgent = void 0;
/*
 * Copyright (c) AXA Group Operations Spain S.A.
 *
 * Licensed under the AXA Group Operations Spain S.A. License (the "License");
 * you may not use this file except in compliance with the License.
 * A copy of the License can be found in the LICENSE.TXT file distributed
 * together with this file.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
const node_object_hash_1 = __importDefault(require("node-object-hash"));
const url_1 = require("url");
const http_agent_1 = require("./http-agent");
const https_agent_1 = require("./https-agent");
const utils_1 = require("./utils");
const hashIt = (0, node_object_hash_1.default)();
const agentStorage = new Map();
/**
 * Create a keepAlive agent that will be create onces per options+target.protocol
 * Also try to gets the proxy from the http/s_proxy env variables
 * @param {string} target - The target to proxy
 * @param {NativeAgentOptions} [options]
 * @returns {HttpsAgent|HttpAgent} The agent depending on the target.
 * @example
 * const { createForeverAgent } = require('@axa/native-proxy-agent');
 * const got = require('got');
 *
 * got('http://myhost.com', { agent : createForeverAgent('http://myhost.com', {cert: 'mycert'}) });
 *
 * // This request will use same agent as they share options and protocol
 * got('http://otherhost.com', { agent : createForeverAgent('http://otherhost.com', {cert: 'mycert'}) });
 */
function createForeverAgent(target, options = {}) {
    if (typeof target !== 'string') {
        throw new Error('Target must be defined.');
    }
    const url = new url_1.URL(target);
    if (options.proxy === undefined) {
        const proxy = (0, utils_1.getProxy)();
        if (proxy) {
            Object.assign(options, {
                proxy
            });
        }
    }
    const key = hashIt.hash({ protocol: url.protocol, ...options });
    if (agentStorage.has(key)) {
        return agentStorage.get(key);
    }
    let agent;
    if (url.protocol === 'https:') {
        agent = new https_agent_1.HttpsAgent({ ...options, keepAlive: true, name: key });
    }
    else if (url.protocol === 'http:') {
        agent = new http_agent_1.HttpAgent({ ...options, keepAlive: true, name: key });
    }
    else {
        throw new Error('Target protocol not supported, should be https or http.');
    }
    agentStorage.set(key, agent);
    return agent;
}
exports.createForeverAgent = createForeverAgent;
/**
 * Create a keepAlive agent that will be create onces per options+http protocol
 * Also try to gets the proxy from the http/s_proxy env variables
 * @param {NativeAgentOptions} [options]
 * @returns {HttpAgent} The agent depending on the target.
 * @example
 * const { createHttpForeverAgent } = require('@axa/native-proxy-agent');
 * const got = require('got');
 *
 * got('http://myhost.com', { agent : createHttpForeverAgent({cert: 'mycert'}) });
 *
 * // This request will use same agent as they share options and protocol
 * got('http://otherhost.com', { agent : createHttpForeverAgent({cert: 'mycert'}) });
 */
function createHttpForeverAgent(options = {}) {
    if (options.proxy === undefined) {
        const proxy = (0, utils_1.getProxy)();
        if (proxy) {
            Object.assign(options, {
                proxy
            });
        }
    }
    const key = hashIt.hash({ protocol: 'http', ...options });
    if (agentStorage.has(key)) {
        return agentStorage.get(key);
    }
    const agent = new http_agent_1.HttpAgent({ ...options, keepAlive: true, name: key });
    agentStorage.set(key, agent);
    return agent;
}
exports.createHttpForeverAgent = createHttpForeverAgent;
/**
 * Create a keepAlive agent that will be create onces per options+https protocol
 * Also try to gets the proxy from the http/s_proxy env variables
 * @param {NativeAgentOptions} [options]
 * @returns {HttpAgent} The agent depending on the target.
 * @example
 * const { createHttpForeverAgent } = require('@axa/native-proxy-agent');
 * const got = require('got');
 *
 * got('https://myhost.com', { agent : createHttpsForeverAgent({cert: 'mycert'}) });
 *
 * // This request will use same agent as they share options and protocol
 * got('https://otherhost.com', { agent : createHttpsForeverAgent({cert: 'mycert'}) });
 */
function createHttpsForeverAgent(options = {}) {
    if (options.proxy === undefined) {
        const proxy = (0, utils_1.getProxy)();
        if (proxy) {
            Object.assign(options, {
                proxy
            });
        }
    }
    const key = hashIt.hash({ protocol: 'https', ...options });
    if (agentStorage.has(key)) {
        return agentStorage.get(key);
    }
    const agent = new https_agent_1.HttpsAgent({ ...options, keepAlive: true, name: key });
    agentStorage.set(key, agent);
    return agent;
}
exports.createHttpsForeverAgent = createHttpsForeverAgent;
//# sourceMappingURL=forever-agent.js.map